'use strict'

var cloneDeep = require('lodash/cloneDeep.js')

module.exports = {
  methods: {
    // copy true props to immutable cache, deep copy props to prop name
    // so that your edition will never change the parent data
    // and watch the data from parent, if it changes change the deep copy
    makeImmutableProps: function (arr) {
      var $this = this;
      var arrType = typeof arr;
      if (arrType === 'string' || arr instanceof Array) {
        arr = typeof arr === 'string' ? [arr] : arr;
      } else {
        throw new TypeError('you should pass a string or an array to make immutable');
      }
      
      var iWatcher = $this.immutableWatcher = $this.immutableWatcher || {};
      arr.forEach(function (item) {
        
        if (iWatcher[item]) {
          console.warn('you turn ' + item + ' to immutable twice, check your code');
          return;
        }
        
        // init immutable props
        $this.$set(item + '_immutable_cache', $this.$get(item));
        $this.$set(item, cloneDeep($this.$get(item)));
        
        // everytime parent data changes change the clone
        iWatcher[item] = $this.$watch(item + '_immutable_cache', function (newVal) {
          $this.$set(item, cloneDeep(newVal));
        }, {deep: true});
        
      });
      
      // if no props need to be made immutable, remove immutableWatcher
      if (iWatcher.length < 1) {
        delete $this.immutableWatcher;
      }
      
    }
  },
  
  created: function () {
    if (this.props_to_immutable) {
      this.makeImmutableProps(this.props_to_immutable);
    }
  },
  
  beforeDestroy: function () {
    if (this.immutableWatcher) {
      this.immutableWatcher.forEach(function (unwatch) {
        unwatch();
      });
    }
  }
}