var Vue = require('vue')
import Vuex from 'vuex'
var vueImmutableHelperMixin = require('../../src/index.js')

var chai = require('chai');
var expect = chai.expect;

Vue.mixin(vueImmutableHelperMixin)

describe("immutable props", function() {
  
  it("parent data should be immutable", function (done) {
    
    var vm = new Vue({
      template: '<test :A="A" :B="B" :C="C"></test>',
      data: {
        A: {a: 1},
        B: 'A',
        C: [1, 2]
      },
      components: {
        test: {
          props: ['A', 'B', 'C'],
          data: function () {
            return {
              props_to_immutable: ['A', 'B', 'C']
            }
          },
          compiled: function () {
            this.A.a = 2;
            this.B = 'B';
            this.C.push(3);
          },
          template: "" +
            "<div>A</div>"
        }
      }
    }).$mount()
    
    Vue.nextTick(function () {
      expect(vm.A.a).to.equal(1);
      expect(vm.B).to.equal('A');
      expect(vm.C.length).to.equal(2);
      done()
    })
    
  })
  
  it("child data update after parent data update", function (done) {
    var vm = new Vue({
      template: '<div><test :A="A" :B="B" :C="C"></test></div>',
      data: {
        A: {a: 1},
        B: 'A',
        C: [1, 2]
      },
      compiled: function () {
        console.log(1)
        this.A.a = 2;
        this.B = 'B';
        this.C.push(3);
      },
      components: {
        test: {
          props: ['A', 'B', 'C'],
          data: function () {
            return {
              props_to_immutable: ['A', 'B', 'C']
            }
          },
          template: "" +
            "{{A.a + B + C.length}}"
        }
      }
    }).$mount()
    
    Vue.nextTick(function () {
      expect(vm.$el.innerHTML).to.equal('2B3');
      done()
    })
  })
  
  it("should work with vuex", function (done) {
    Vue.use(Vuex)
    
    var store = new Vuex.Store({
      state: {obj: {a: 1}}
    })
    
    var vm = new Vue({
      data: {
        obj: store.state.obj,
        props_to_immutable: 'obj'
      },
      template: "A",
      beforeCompile: function () {
        this.obj.a = 2
      }
    }).$mount()
    
    expect(vm.obj.a).to.equal(2)
    expect(store.state.obj.a).to.equal(1)
    done()
    
  })
  
})